/*
 * Copyright (C) 2023, KylinSoft Co., Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 *
**/
#ifndef GETBRIGHTNESSTHREAD_H
#define GETBRIGHTNESSTHREAD_H
#include <QThread>


class GetBrightnessThread : public QThread{
    Q_OBJECT
public:
    GetBrightnessThread(QString edidHash, QString i2cBus);
    ~GetBrightnessThread();
    void setExit(bool b);
protected:
    void run()override;
private:
    QString edidHash;
    QString i2cBus;
    bool exitFlag;
Q_SIGNALS:
    void getBrightnessFinished(int value);
};


#endif // GETBRIGHTNESSTHREAD_H
