/*
 * Copyright (C) 2023, KylinSoft Co., Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 *
**/
#ifndef SIMPLETHREAD_H
#define SIMPLETHREAD_H

#include <QThread>
#include <QMap>
#include <QListWidgetItem>

class SimpleThread : public QThread
{
    Q_OBJECT

public:
    explicit SimpleThread(QMap<QString, QMap<QString, QString>> _wpinfo, QObject * parent);
    ~SimpleThread();

protected:
    void run();

private:
    QMap<QString, QMap<QString, QString>> wallpaperinfosMap;

Q_SIGNALS:
    void widgetItemCreate(QPixmap pixmap, QString filename);


};

#endif // SIMPLETHREAD_H
