/** Used in class PopupMenu **/

QHeaderView,
QHeaderView::section,
QListCornerButton::section {
  background-color: transparent;
}

QListView {
  font-size: 12px;
  border: 0px;
  border-radius: 4px;
  margin: 4px 0px;
  padding: 4px;
  background: palette(base);

  /* make the selection span the entire width of the view */
  show-decoration-selected: 0;
}

QListView::item {
  background: transparent;
  height: 24px;
  padding: 0px;
  margin: 0px;
  outline: none;
  border-bottom: 1px solid rgba(0, 0, 0, 0.05);
}
