/*
 * Copyright (C) 2023, KylinSoft Co., Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 *
**/
#include "type.h"

const QDBusArgument &operator<<(QDBusArgument &argument, const QStringPair &string)
{
    argument.beginStructure();
    argument << string.name;
    argument << string.key;
    argument << string.mediaKey;
    argument.endStructure();
    return argument;
}

const QDBusArgument &operator>>(const QDBusArgument &argument, QStringPair &string)
{
    argument.beginStructure();
    argument >> string.name;
    argument >> string.key;
    argument >> string.mediaKey;
    argument.endStructure();
    return argument;
}

const QDBusArgument &operator<<(QDBusArgument &argument, const KeyEntry &key)
{
    argument.beginStructure();
    argument << key.gsSchema;
    argument << key.keyStr;
    argument << key.valueStr;
    argument << key.descStr;
    argument << key.gsPath;
    argument << key.nameStr;
    argument << key.bindingStr;
    argument << key.actionStr;
    argument.endStructure();
    return argument;
}

const QDBusArgument &operator>>(const QDBusArgument &argument, KeyEntry &key)
{
    argument.beginStructure();
    argument >> key.gsSchema;
    argument >> key.keyStr;
    argument >> key.valueStr;
    argument >> key.descStr;
    argument >> key.gsPath;
    argument >> key.nameStr;
    argument >> key.bindingStr;
    argument >> key.actionStr;
    argument.endStructure();
    return argument;
}
