/*
    Copyright 2019 Harald Sitter <sitter@kde.org>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of
    the License or (at your option) version 3 or any later version
    accepted by the membership of KDE e.V. (or its successor approved
    by the membership of KDE e.V.), which shall act as a proxy
    defined in Section 14 of version 3 of the license.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/* config-workspace.h.  Generated by cmake from config-workspace.h.cmake  */

/* #undef HAVE_QIMAGEBLITZ */

/* Define if you have DPMS support */
#define HAVE_DPMS 1

/* Define if you have the DPMSCapable prototype in <X11/extensions/dpms.h> */
/* #undef HAVE_DPMSCAPABLE_PROTO */

/* Define if you have the DPMSInfo prototype in <X11/extensions/dpms.h> */
/* #undef HAVE_DPMSINFO_PROTO */

/* Defines if your system has the libfontconfig library */
#define HAVE_FONTCONFIG 1

/* Defines if your system has the freetype library */
#define HAVE_FREETYPE 1

/* Define if you have gethostname */
/* #undef HAVE_GETHOSTNAME */

/* Define if you have the gethostname prototype */
/* #undef HAVE_GETHOSTNAME_PROTO */

/* Define to 1 if you have the `getpeereid' function. */
/* #undef HAVE_GETPEEREID */

/* Defines if you have Solaris' libkstat */
/* #undef HAVE_KSTAT */

/* Define if you have long long as datatype */
/* #undef HAVE_LONG_LONG */

/* Define to 1 if you have the `nice' function. */
/* #undef HAVE_NICE */

/* Define to 1 if you have the <sasl.h> header file. */
/* #undef HAVE_SASL_H */

/* Define to 1 if you have the <sasl/sasl.h> header file. */
/* #undef HAVE_SASL_SASL_H */

/* Define to 1 if you have the `setpriority' function. */
#define HAVE_SETPRIORITY 1

/* Define to 1 if you have the `sigaction' function. */
/* #undef HAVE_SIGACTION */

/* Define to 1 if you have the `sigset' function. */
/* #undef HAVE_SIGSET */

/* Define to 1 if you have statvfs */
#define HAVE_STATVFS 1

/* Define to 1 if you have the <string.h> header file. */
/* #undef HAVE_STRING_H */

/* Define if you have the struct ucred */
/* #undef HAVE_STRUCT_UCRED */

/* Define to 1 if you have the <sys/loadavg.h> header file. */
/* #undef HAVE_SYS_LOADAVG_H */

/* Define to 1 if you have the <sys/mount.h> header file. */
#define HAVE_SYS_MOUNT_H 1

/* Define to 1 if you have the <sys/param.h> header file. */
#define HAVE_SYS_PARAM_H 1

/* Define to 1 if you have the <sys/statfs.h> header file. */
#define HAVE_SYS_STATFS_H 1

/* Define to 1 if you have the <sys/statvfs.h> header file. */
#define HAVE_SYS_STATVFS_H 1

/* Define to 1 if you have statfs(). */
#define HAVE_STATFS 1

/* Define to 1 if you have the <sys/select.h> header file. */
/* #undef HAVE_SYS_SELECT_H */

/* Define to 1 if you have the <sys/socket.h> header file. */
/* #undef HAVE_SYS_SOCKET_H */

/* Define to 1 if you have the <sys/time.h> header file. */
#define HAVE_SYS_TIME_H 1

/* Define to 1 if you have the <sys/types.h> header file. */
/* #undef HAVE_SYS_TYPES_H */

/* Define to 1 if you have the <sys/vfs.h> header file. */
#define HAVE_SYS_VFS_H 1

/* Define to 1 if you have the <sys/wait.h> header file. */
/* #undef HAVE_SYS_WAIT_H */

/* Define to 1 if you have the <unistd.h> header file. */
#define HAVE_UNISTD_H 1

/* Define to 1 if you have the <stdint.h> header file. */
#define HAVE_STDINT_H 1

/* Define to 1 if you have the <malloc.h> header file. */
#define HAVE_MALLOC_H 1

/* Define if you have unsetenv */
/* #undef HAVE_UNSETENV */

/* Define if you have the unsetenv prototype */
/* #undef HAVE_UNSETENV_PROTO */

/* Define if you have usleep */
/* #undef HAVE_USLEEP */

/* Define if you have the usleep prototype */
/* #undef HAVE_USLEEP_PROTO */

/* Define to 1 if you have the `vsnprintf' function. */
/* #undef HAVE_VSNPRINTF */

/* Define to 1 if you have the Wayland libraries. */
/* #undef WAYLAND_FOUND */

/* KDE's default home directory */
/* #undef KDE_DEFAULT_HOME */

/* KDE's binaries directory */
#define KDE_BINDIR "bin"

/* KDE's configuration directory */
#define KDE_CONFDIR "/etc/xdg"

/* KDE's static data directory */
#define KDE_DATADIR "share"

/* Define where your java executable is */
#undef PATH_JAVA

/* Define to 1 if you can safely include both <sys/time.h> and <time.h>. */
/* #undef TIME_WITH_SYS_TIME */

/* xkb resources directory */
#define XKBDIR "/usr/share/X11/xkb"

/* KWin binary name */
#define KWIN_BIN "kwin_x11"

/* Number of bits in a file offset, on hosts where this is settable. */
#define _FILE_OFFSET_BITS 64

/* Define 1 if the Breeze window decoration was found */
#define HAVE_BREEZE_DECO 1

#ifdef HAVE_BREEZE_DECO
#define BREEZE_KDECORATION_PLUGIN_ID "org.kde.breeze"
#endif

/*
 * On HP-UX, the declaration of vsnprintf() is needed every time !
 */

/* type to use in place of socklen_t if not defined */
#define kde_socklen_t socklen_t

#define WORKSPACE_VERSION_STRING "5.18.3"
