/*
 * Copyright (C) 2023, KylinSoft Co., Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 *
**/
#ifndef POPLISTDELEGATE_H
#define POPLISTDELEGATE_H

#include <QStyledItemDelegate>
#include <QPainter>
#include <QColor>

// 委托类，使用当前的样式绘制list
class PopListDelegate : public QStyledItemDelegate
{
    Q_OBJECT
public:
    PopListDelegate(QWidget* parent = nullptr);

protected:
    void paint(QPainter* painter,
               const QStyleOptionViewItem& option,
               const QModelIndex& index) const override;
};

#endif // POPLISTDELEGATE_H
